import 'dart:async';

import 'package:http/http.dart';

class PingService {
  late Stream<bool>? _connection;
  final String urlTarget;
  final Duration timeout;
  final Client _httpClient = Client();
  int _clients = 0;

  PingService({
    this.urlTarget = 'https://www.google.com/',
    this.timeout = const Duration(seconds: 3),
  }) {
    _restartStream();
  }

  Stream<bool> _pingTick() async* {
    while (true) {
      await Future.delayed(timeout);
      bool hasConnection = await _pingServer();
      yield hasConnection;
    }
  }

  _pauseStream() {
    _connection = null;
  }

  _restartStream() {
    _connection = _pingTick().asBroadcastStream(
      onCancel: (_) {
        _clients--;
        if (_clients == 0) {
          _pauseStream();
        }
      },
      onListen: (_) {
        _clients++;
      },
    );
  }

  Future<bool> _pingServer() async {
    try {
      Uri url = Uri.parse(urlTarget);

      Response response =
          await _httpClient.get(url).timeout(timeout, onTimeout: () {
        throw ReachServerFailException();
      });

      return response.statusCode == 200;
    } on Exception catch (_) {
      return false;
    }
  }

  StreamSubscription getSubscription(
      {required Function(bool) onTrue, Function(bool)? onFalse}) {
    if (_connection == null) {
      _restartStream();
    }

    return _connection!.listen(
      (hasConnection) {
        onFalse == null
            ? onTrue(hasConnection)
            : hasConnection
                ? onTrue(hasConnection)
                : onFalse.call(hasConnection);
      },
    );
  }
}

class ReachServerFailException implements Exception {}
